.bold {font-weight: bold}
.date {float: right; font-weight: 200; margin-right: 3%}
.footer {font-size: 80%}
.italic {font-style: italic}
.new {float: right; font-weight: 600; margin-right: 3%}
.underline {text-decoration: underline}
.update {float: right; font-weight: 400; margin-right: 3%}

a:active {color: #ff00ff; font-weight: bold}
a:link {color: #5555ff}
a:visited {color: #aa00aa}

a.darkgrey:active {color: #ff00ff; font-weight: bold; background-color: #333333}

body {color: #ffffff; background-color: #000000;
      background-repeat: no-repeat;
      background-position: top right;
     }

li.black {color: #ffffff; background-color: #000000}
li.darkergrey {color: #ffffff; background-color: #151515}
li.darkgrey {color: #ffffff; background-color: #333333}
li.grey {color: #000000; background-color: #888888}
li.white {color: #000000; background-color: #ffffff}

ol li {list-style: upper-roman}
ol ol li {list-style: upper-alpha}
ol ol ol li {list-style: decimal}
ol ol ol ol li {list-style: lower-alpha}

p.black {margin-left: 3%; margin-right: 3%; color: #ffffff; background-color: #000000}
p.comment {margin-left: 3%; margin-right: 50%; color: #000000; background-color: #888888}
p.darkergrey {margin-left: 3%; margin-right: 3%; color: #ffffff; background-color: #151515}
p.darkgrey {margin-left: 3%; margin-right: 3%; color: #ffffff; background-color: #333333}
p.main {margin-left: 3%; margin-right: 3%}

table.main {border: 0 none}

tr.header {color: #ffffff; background-color: #333333}
tr.subheader {color: #ffffff; background-color: #222222}
tr.comment {color: #ffffff; background-color: #333333}

ul {list-style-type: none; margin-left: 3%; margin-right: 3%}
ul.black {color: #ffffff; background-color: #000000}
ul.darkergrey {color: #ffffff; background-color: #151515}
ul.darkgrey {color: #ffffff; background-color: #333333}
ul.grey {color: #000000; background-color: #888888}
ul.white {color: #000000; background-color: #ffffff}
